package test;

import jmmc;
import java.io.*;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: Performs semi-automated system tests<br>
 * Tests the semantic checking automatically and compiles the class files, but the class
 * files must be manually run to check we get the desired result</p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public class SystemTests
{
    private static PrintWriter err = new PrintWriter(System.out, true);

    public SystemTests()
    {
    }

    //run all the system tests (ie compile all the files in the tests directory (except the samples))
    private static void runTests() throws Exception
    {
        int totalTests = 0;
        int passedTests = 0;

        //these tests should all pass semantic analysis
        File correctDir = new File("./tests/correct");
        String[] correctFiles = correctDir.list();  //get a directory listing
        for (int i = 0; i < correctFiles.length; ++i)
        {
            ++totalTests;
            if (runTest(correctFiles[i], "./tests/correct/"))
                ++passedTests;
        }
        File sampleDir = new File("./tests/samples");
        String[] sampleFiles = sampleDir.list();  //get a directory listing
        for (int i = 0; i < sampleFiles.length; ++i)
        {
            ++totalTests;
            if (runTest(sampleFiles[i], "./tests/samples/"))
                ++passedTests;
        }

        //these tests should all fail semantic analysis
        File badDir = new File("./tests/errors");
        String[] badFiles = badDir.list();  //get a directory listing
        for (int i = 0; i < badFiles.length; ++i)
        {
            ++totalTests;
            if (!runTest(badFiles[i], "./tests/errors/"))
                ++passedTests;
        }

        System.out.println("\nPassed " + passedTests + " out of " + totalTests + " system tests.");
    }

    //compile a file, returns true if the file compiled, flase otherwise
    private static boolean runTest(String sourceFileName, String path) throws Exception
    {
        //open a reader on the file
        File sourceFile = new File(path + sourceFileName);
        BufferedReader input = new BufferedReader(new FileReader(sourceFile));

        System.out.println("\n" + sourceFileName + ":");

        //compile it and check the result
        boolean result = jmmc.compile(err, input, sourceFileName, sourceFileName.substring(0, sourceFileName.indexOf('.')), null, true, -1, -1) != null;

        input.close();
        return result;
    }

    public static void main(String[] args) throws Exception
    {
        runTests();
    }
}